//========================================================================
// Author: Jason Olkonen
// seniorproject [at] longyear [dot] nmu [dot] edu
// Date: 4/21/03
// Compiler: gcc version 3.2 20020903 (Red Hat Linux 8.0 3.2-7)
// Operating System: Red Hat Linux 8.0
// Copyright Jason Olkonen - 4/21/03 - free for noncommercial use
// commercial use prohibited, unless you hire me..... I need a job ;-)
// ----------------------------------------------------------------------
// File: Align.cpp
// Senior Project in Network Computing at Northern Michigan University
// Autonomous 802.11b Antenna Alignment System
//========================================================================

#include <iostream>
#include <fstream>
#include <sstream>
#include <stdio.h>
#include <string>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <stdlib.h>
#include <unistd.h>
#include <ncurses.h>
#include <time.h>
#include "/usr/local/include/iwlib.h"//used to get the wireless stats
#include "MiniSSCMotionController.hpp"//controls the servos
using namespace std;
#define MAXAZW 141
#define MAXELW 61
#define LEFT -70
#define RIGHT 70
#define DOWN -30
#define UP 30
static int newinfo(int skfd, char* interface, struct wireless_info* info);
static void dostats(int skfd, char* interface, struct wireless_info* info, const double azp, const double elp, int qualarray [] [MAXELW], int sigarray [] [MAXELW], int noisearray [] [MAXELW]);
void findbest(int qualarray [] [MAXELW], int sigrray [] [MAXELW], int noisearray [] [MAXELW],double* azp, double* elp);
static void clearstats (int qualarray[] [MAXELW], int sigarray [] [MAXELW], int noisearray [] [MAXELW]);


/*********************************************************************
MAIN
*********************************************************************/

int main()
{
	MiniSSCMotionController lynxPort("/dev/usb/ttyUSB0", 9600, 180, 0);
	// write the setpoint queue
	lynxPort.setSpeedFactor(3);
	const int az = 0;
	const int el = 1;
	double azp = 0;
	double elp = 0;
	int quit = 0;
	int row, col;
	getmaxyx(stdscr, row, col);
	char* ifname = "eth1";
	struct wireless_info info;
	struct wireless_info* infop = &info;
	int qualarray [MAXAZW] [MAXELW] = {{0}};
	int sigarray [MAXAZW] [MAXELW] = {{0}};
	int noisearray [MAXAZW] [MAXELW] = {{0}};
	int sleeptime = 100000;
	ifstream readfile;
	initscr();
	keypad(stdscr, TRUE); //tell curses that there is a keypad
	noecho();//don't echo any typed characters
	scrollok(stdscr, TRUE);
	curs_set(0);//make the cursor invisible
	int posneg = 0;
	int flip = 0;
	int skfd; // generic raw socket desc

	// Create a channel to the NET kernel
	if((skfd = iw_sockets_open()) < 0){
	      perror("socket");
	      exit(-1);
    }
do
{
	mvprintw(1,2, "Press the following keys:\n  The arrow keys move the antenna\n  F1 starts a 5x5 Scan\n  F2 starts a Patch Scan\n  F3 starts a Random Scan\n  F12 clears all of the stats from ram\n  b finds the best signal out of the positions tried\n  c centers the antenna\n  q quits the program\n");
	wborder(stdscr, 0, 0, 0, 0, 0, 0, 0, 0);
	int ch = getch();
switch (ch) {
	case 'q': // user wants to quit
	{
		quit=1;
		break;
	}
	case 'b': // find the best signal out of sigarray
	{
		findbest(qualarray, sigarray, noisearray, &azp, &elp);
		lynxPort.moveTo(el, elp);
		lynxPort.moveTo(az, azp);
		usleep(sleeptime);//to make certain that the antenna got to the position
		dostats(skfd,ifname, infop, azp, elp, qualarray, sigarray, noisearray);
		break;
	}
	case 'c': // user centered antenna
	{
		azp=0;
		elp=0;
		lynxPort.moveTo(el, elp);
		lynxPort.moveTo(az, azp);
		usleep(sleeptime);//to make certain that the antenna got to the position
		dostats(skfd,ifname, infop, azp, elp, qualarray, sigarray, noisearray);
		break;
	}
	case KEY_UP:  // user pressed up arrow key
	{
		if(elp<=UP-2)
		{
			elp+=2;
			lynxPort.moveTo(el, elp);
			usleep(sleeptime);//to make certain that the antenna got to the position
			dostats(skfd,ifname, infop, azp, elp, qualarray, sigarray, noisearray);
		}
		break;
	}
	case KEY_DOWN:  // user pressed down arrow key
	{
		if(elp>=DOWN+2)
		{
			elp-=2;
			lynxPort.moveTo(el, elp);
			usleep(sleeptime);//to make certain that the antenna got to the position
			dostats(skfd,ifname, infop, azp, elp, qualarray, sigarray, noisearray);
		}
		break;
	}
	case KEY_LEFT:  // user pressed left arrow key
	{
		if(azp>=LEFT+2)
		{
			azp-=2;
			lynxPort.moveTo(az, azp);
			usleep(sleeptime);//to make certain that the antenna got to the position
			dostats(skfd,ifname, infop, azp, elp, qualarray, sigarray, noisearray);
		}
		break;
	}
	case KEY_RIGHT:  // user pressed right arrow key
	{
    	if(azp<=RIGHT-2)
	    {
			azp+=2;
	     	lynxPort.moveTo(az, azp);
	     	usleep(sleeptime);//to make certain that the antenna got to the position
			dostats(skfd,ifname, infop, azp, elp, qualarray, sigarray, noisearray);
	 	}
        break;
	 }
	case KEY_F(1): // user enabled 5x5 scan
	{
		clearstats(qualarray, sigarray, noisearray);
		mvprintw(11,2, "--->5x5 Scan<---\n");
		wrefresh(stdscr);
		azp= (double)LEFT;
		elp= (double)DOWN;
		flip = 0;//when flip == 0, go right: when flip ==1 go left
		while(elp<=UP)
		{
			lynxPort.moveTo(el, elp);
			lynxPort.moveTo(az, azp);
			//if just starting out then give servos time to get to the right spot
			if (azp==(double)LEFT && elp==(double)DOWN)
				sleep(3);
			usleep(300000);
			dostats(skfd, ifname, infop, azp, elp, qualarray, sigarray, noisearray);
			if(azp==(double)RIGHT && flip!=1)
				{flip=1; elp+=5;}
			else if (azp==(double)LEFT && flip!=0 && elp!=(double)DOWN)
				{flip=0; elp+=5;}
			else if (flip==0)
				azp+=5;
			else if (flip==1)
				azp-=5;
		}
		//set azp and elp to the best possible signal level
		findbest(qualarray, sigarray, noisearray, &azp, &elp);
		lynxPort.moveTo(el, elp);
		lynxPort.moveTo(az, azp);
		sleep(2);
		dostats(skfd, ifname, infop, azp, elp, qualarray, sigarray, noisearray);
		mvprintw(11,2, "                    \n");
		wrefresh(stdscr);
		break;
	}
	case  KEY_F(2): //user enabled patch scan
	{
		clearstats(qualarray, sigarray, noisearray);
		mvprintw(11,2, "--->Patch Scan<---\n");
		wrefresh(stdscr);
		azp= (double)LEFT;
		elp= (double)DOWN;
		double left = azp;
		double down = elp;
		double right = (double)RIGHT;
		double up = (double)UP;
		flip = 0;//when flip == 0, go right: when flip ==1 go left
		double stepsize=20;
		for(int step=0; step<=1; step++)
		{
		while(elp<=up)
		{
			lynxPort.moveTo(el, elp);
			lynxPort.moveTo(az, azp);
			//if just starting out then give servos time to get to the right spot
			if (azp==left && elp==down)
				sleep(3);
			usleep(450000);
			dostats(skfd, ifname, infop, azp, elp, qualarray, sigarray, noisearray);
			if(azp==right && flip!=1)
				{flip=1; elp+=stepsize;}
			else if (azp==left && flip!=0 && elp!=down)
				{flip=0; elp+=stepsize;}
			else if (flip==0)
				azp+=stepsize;
			else if (flip==1)
				azp-=stepsize;
		}
		findbest(qualarray, sigarray, noisearray, &azp, &elp);
		//check to see if we are really close to the far left or right
		if (azp+10>(double)RIGHT)
			{right=(double)RIGHT; left=right-20; azp=left;}
		else if (azp-10<(double)LEFT)
			{left=(double)LEFT; right=left+20; azp=left;}
		else
			{right=azp+10; left=azp-10;}
		//check to see if we are really close to the top or bottom
		if (elp+10>(double)UP)
			{up=(double)UP; down=up-20; elp=down;}
		else if (elp-10<(double)DOWN)
			{down=(double)DOWN; up=down+20; elp=down;}
		else
			{up=elp+10; down=elp-10; elp=down;}

		stepsize=5;
		}
		findbest(qualarray, sigarray, noisearray, &azp, &elp);
		lynxPort.moveTo(az, azp);
		lynxPort.moveTo(el, elp);
		sleep(2);
		dostats(skfd, ifname, infop, azp, elp, qualarray, sigarray, noisearray);
		mvprintw(11,2, "                    \n");
		wrefresh(stdscr);
		break;

	}
	case KEY_F(3): //user enabled a random scan
	{
		clearstats(qualarray, sigarray, noisearray);
		mvprintw(11,2, "--->Random Scan<---\n");
		wrefresh(stdscr);
		for (int step=0; step<31; step++){
		srand(time(0));//seed rand
		posneg = (int) (2.0*rand()/(RAND_MAX));
		if (posneg) {azp = 1+(double) (70.0*rand()/(RAND_MAX));}
		else {azp = 1+(double) (-70.0*rand()/(RAND_MAX));}
		posneg = (int) (2.0*rand()/(RAND_MAX));
		if (posneg) {elp = 1+(double) (30.0*rand()/(RAND_MAX));}
		else {elp = 1+(double) (-30.0*rand()/(RAND_MAX));}

		lynxPort.moveTo(az, azp);
		lynxPort.moveTo(el, elp);
		sleep(2);
		dostats(skfd,ifname, infop, azp, elp, qualarray, sigarray, noisearray);
		}
		findbest(qualarray, sigarray, noisearray, &azp, &elp);
		lynxPort.moveTo(az, azp);
		lynxPort.moveTo(el, elp);
		sleep(2);
		dostats(skfd, ifname, infop, azp, elp, qualarray, sigarray, noisearray);
		mvprintw(11,2, "                    \n");
		wrefresh(stdscr);
		break;
	}
	case KEY_F(12): //clear the array holding the readings
	{
		clearstats(qualarray, sigarray, noisearray);
		break;
	}
}
 } while(quit != 1);
	lynxPort.quit();
	readfile.close();
	endwin();
	return 0;
}

//**********************END MAIN **********************************************



/********************************************************************
Prints stats and saves stats to array
********************************************************************/

static void dostats(int skfd, char* interface, struct wireless_info* info, const double azp, const double elp, int qualarray [] [MAXELW], int sigarray [] [MAXELW], int noisearray [] [MAXELW])
{
	//prevent any values outside of the acceptable range from being computed
	if (azp>=(double)LEFT && azp<=(double)RIGHT && elp>=(double)DOWN && elp<=(double)UP){
		//convert the azimuth and elevation from -70to70 and -30to30 to 0to141 and 0to61 respectively
		int azpi = RIGHT+static_cast<int>(azp);
		int elpi = UP+static_cast<int>(elp);
		char* buf = new char [100];
		int quality = 0;
		int signal = 0;
		int noise = 0;
		int statspos = 12;
		struct wireless_info* infop = info;
		for (int loopstats=0; loopstats<=100; loopstats++){
			newinfo(skfd, interface, info);
			iw_print_stats(buf, &infop->stats.qual, &infop->range, infop->has_range);
			quality+=atoi(buf);
			signal+=atoi(buf+16);
			noise+=atoi(buf+38);
		}
	qualarray[azpi][elpi]= quality/100;
	sigarray[azpi][elpi]= signal/100;
	noisearray[azpi][elpi]= noise/100;
	mvprintw(statspos, 2, "Azimuth: %d\n", static_cast<int>(azp));
	mvprintw(statspos+1, 2, "Elevation: %d\n", static_cast<int>(elp));
	iw_print_bitrate(buf, infop->bitrate.value);
	mvprintw(statspos+2, 2, "BitRate:%c%s", (infop->bitrate.fixed ? '=' : ':'), buf);
	mvprintw(statspos+3, 2, "Link Quality | Signal level in dBm | Noise level in dBm\n");
	mvprintw(statspos+4, 2, "      %d               %d                  %d ", qualarray[azpi][elpi], sigarray[azpi][elpi], noisearray[azpi][elpi]);
	wrefresh(stdscr);
	}
}
/********************************************************************
Find the best signal out of the array and writes stats to file
********************************************************************/

void findbest(int qualarray[] [MAXELW], int sigarray [] [MAXELW], int noisearray [] [MAXELW], double* azp, double* elp)
{
		int row=0;
		int column=0;
		int azbest=0;
		int elbest=0;
		int signalbest = -1000;
		int step;
		char* buf = new char [100];
		int similar=0;
		ofstream writefile;
		writefile.open("stats.csv");
		if (! writefile.is_open())
			{cerr << "Error opening the file \n";exit (1);}
		for (column = 0; column<(int)MAXELW; column++){
			for (row=0;row<(int)MAXAZW; row++){
				sprintf(buf, "%d,",row+LEFT);
				step = strlen(buf);
				sprintf(buf+step, "%d,",column+DOWN);
				step = strlen(buf);
				sprintf(buf+step, "%d,", qualarray[row][column]);
				step = strlen(buf);
				sprintf(buf+step, "%d,", sigarray[row][column]);
				step = strlen(buf);
				sprintf(buf+step, "%d\n", noisearray[row][column]);
				writefile.write(buf, strlen(buf));
				if (sigarray[row][column]==signalbest)
					{similar++;}
				else if (sigarray[row][column]> signalbest && sigarray[row][column] < 0){
					signalbest = sigarray[row][column];
					azbest=row;
					elbest=column;
					similar=0;
				}
		    }
		}
	writefile.close();
	mvprintw(17,2, "The best signal was found at azimuth:%d  and elevation:%d with a signal strength of %d dBm\n", azbest+LEFT, elbest+DOWN, signalbest);
	mvprintw(18,2, "The same signal strength was found at %d other spot(s)\n", similar);
	//convert the azimuth and elevation from 0to141 and 0to61 to -70to70 and -30to30 respectively
	*azp = static_cast<double>(azbest)+LEFT;
	*elp = static_cast<double>(elbest)+DOWN;
}

/*****************************************************************************
Clear all of the stats out of ram
*****************************************************************************/
static void clearstats (int qualarray[] [MAXELW], int sigarray [] [MAXELW], int noisearray [] [MAXELW])
{
	int row=0;
	int column=0;
	for (column = 0; column<(int)MAXELW; column++){
			for (row=0;row<(int)MAXAZW; row++)
				{qualarray[row][column]=0;sigarray[row][column]=0;noisearray[row][column]=0;}
	    }
	 mvprintw(11,2, "Stats Cleared\n");
	 wrefresh(stdscr);
	 sleep(2);//so the user can see the message
	 mvprintw(11,2, "                  ");
	 wrefresh(stdscr);
}

//Get wireless info & stats from the device driver
static int newinfo(int skfd, char* interface, struct wireless_info* info)
{
  struct iwreq wrq;

  memset((char *) info, 0, sizeof(struct wireless_info));

  //Get wireless name & check to see if the interface exists
  if(iw_get_ext(skfd, interface, SIOCGIWNAME, &wrq) < 0)
    {
      struct ifreq ifr;
      strcpy(ifr.ifr_name, interface);
      if(ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0)
	return(-ENODEV);
      else
	return(-ENOTSUP);
    }
  else
    {
      strncpy(info->name, wrq.u.name, IFNAMSIZ);
      info->name[IFNAMSIZ] = '\0';
    }

 	//Get ranges
  if(iw_get_range_info(skfd, interface, &(info->range)) >= 0)
    info->has_range = 1;

	//Get network ID
  if(iw_get_ext(skfd, interface, SIOCGIWNWID, &wrq) >= 0)
    {
      info->has_nwid = 1;
      memcpy(&(info->nwid), &(wrq.u.nwid), sizeof(iwparam));
    }

	//Get bit rate
  if(iw_get_ext(skfd, interface, SIOCGIWRATE, &wrq) >= 0)
    {
      info->has_bitrate = 1;
      memcpy(&(info->bitrate), &(wrq.u.bitrate), sizeof(iwparam));
    }
  //Get stats from device driver
  if(iw_get_stats(skfd, interface, &(info->stats)) >= 0)
    {
      info->has_stats = 1;
    }

  return(0);
}
