//========================================================================
// Project: none
// ----------------------------------------------------------------------
// Package: Mini SSC-II communication module
// Authors: Vilas Kumar Chitrakaran (cvilas@ces.clemson.edu)
// Start Date: Sat May 18 06:46:12 edt 2002
// Compiler: GNU GCC 2.96
// Operating System: GNU/Linux
// ----------------------------------------------------------------------
// File: MiniSSCPort.cpp
// Implementation of the class MiniSSCPort.
// See MiniSSCPort.hpp for more details.
//========================================================================

// ----- Project Includes -----
#include "MiniSSCPort.hpp"

//========================================================================
// MiniSSCPort::MiniSSCPort Constructor of the MiniSSCPort class
//========================================================================

MiniSSCPort::MiniSSCPort (char *portName, int baud )
{

  // set the serial port name and baud rate.
  d_portName = portName;
  d_baudRate = baud;

  // don't initialize a port yet
  d_serialPort = 0;

  // Now set all the parameters for the port, if set
  if (portName)
	reset();
}


//========================================================================
// MiniSSCPort::~MiniSSCPort
//========================================================================

MiniSSCPort::~MiniSSCPort ()
{

#ifdef INCLUDEDEBUGINFO
  cout << "[MiniSSCPort::~MiniSSCPort] : Closing the serial port." << endl;
#endif

  if (d_serialPort)
  	close (d_serialPort);
}


//========================================================================
// MiniSSCPort::reset
//========================================================================

void MiniSSCPort::reset()
{
  char resetWord[3];

  // close the current serial port if open
  	if (d_serialPort)
  		close (d_serialPort);

   // The baud rate termios constants. Real pain in the ass, but what tbe
  // heck.
  int baudRate;
  switch(d_baudRate)
  {
  	case 2400: baudRate = B2400; break;
  	case 9600: baudRate = B9600; break;
  	default:
  	{
  		cout << "[MiniSSCPort::resetPort] : Invalid baudrate specified. "
  		     << "Using 2400 baud setting instead." << endl;
  		baudRate = B2400;
  	}
  }

  struct termios p_terminalOptions;
  // The POSIX terminal control structure.

  d_status = e_ok;
  // Set the status OK. Nothing interesting has happened yet.

  // now open the serial port
  if ((d_serialPort = open (d_portName, O_RDWR)) == -1)
  {
  	cout << "[MiniSSCPort::resetPort] : Could not open serial port "
  	     << d_portName << "."<< endl;
    	d_status=e_commError;
    	std::exit(-1);
  }


  // Set the parameters for the serial port.
  tcgetattr (d_serialPort, &p_terminalOptions); // get the current attributes
  cfmakeraw(&p_terminalOptions); // set up for binary IO
  p_terminalOptions.c_cflag &= ~(IXON|IXOFF); // disable flow control
  cfsetispeed (&p_terminalOptions, baudRate); // set input baud rate
  cfsetospeed (&p_terminalOptions, baudRate); // set output baud rate
  tcsetattr (d_serialPort, TCSANOW, &p_terminalOptions); // set new attributes

#ifdef INCLUDEDEBUGINFO
  // Check the actual settings of the serial port.
  memset(&p_terminalOptions, 0, sizeof(p_terminalOptions));
  tcgetattr(d_serialPort, &p_terminalOptions);

  // Get the actual baudrate.
  speed_t speed = cfgetospeed( &p_terminalOptions);

  cout << endl
       << "[MiniSSCPort::resetPort] : Mini SSC-II Serial Port Settings"
       << endl
       << "Port : " << d_portName << endl
       << "Baud rate : " << speed << endl;
#endif

  // Initialize SSC outputs
  resetWord[0] = 0xFF;
  resetWord[2] = 0x80;
  for (int channel = 0; channel < 8; channel++)
  {
  	resetWord[1] = (char)channel;
  	writeToPort(resetWord, 3);
  }

  d_status = e_ok;
}


//========================================================================
// MiniSSCPort - Definitions over.
//========================================================================
